const r = require('rethinkdb')
const debug = require('debug')('rethinkdb-populate')

// normalize ['a', 'b'] or {a: {…}, b: {…}} to [{name: 'a', …}, {name: 'b', …}]
const toArray = xs =>
  // xs is falsy -> empty array
  !xs ? []
  // xs is string -> single element array with just a name
  : typeof xs == 'string' ? [{name: xs}]
  // xs is array -> just make sure all elements are objects with a name
  : Array.isArray(xs) ? xs.map(x => typeof x != 'object' ? {name: x} : x)
  // else -> take object's entries and use key as the name
  : Object.entries(xs).map(([name, x]) => Object.assign(x, {name}))

// remove fields from object
const without = (obj, fields) => Object.entries(obj)
  .filter(([key]) => !fields.includes(key))
  .reduce((obj, [key, value]) => Object.assign(obj, {[key]: value}), {})

module.exports = async function populate(db, tables) {

  // for every table...
  await Promise.all(toArray(tables).map(async tbl => {

    const opts = without(tbl, ['name', 'indices', 'data'])

    // create table
    await r.tableCreate(tbl.name, opts).run(db)
      .then(() => { debug(`Created table ${tbl.name}`) })
      .catch(() => {})

    // for every index in table...
    await Promise.all(toArray(tbl.indices).map(async ind => {

      const fields = toArray(ind.fields || ind.name).map(fld => r.row(fld.name))
      const opts = without(ind, ['name'])

      // create index
      await r.table(tbl.name).indexCreate(ind.name, fields, opts).run(db)
        .then(() => { debug(`Created index ${ind.name} in ${tbl.name}`) })
        .catch(() => {})

    }))

    // for every initial document with an id...
    await Promise.all(Array.prototype.concat(tbl.data || [])
      .filter(doc => doc.id).map(async doc => {

        // insert doc
        await r.table(tbl.name).insert(doc).run(db)
          .then(res => { if (res.first_error) throw Error(res.first_error) })
          .then(() => { debug(`Inserted ${doc.id} into ${tbl.name}`) })
          .catch(() => {})

      })
    )

  }))

}
